# cannot be used for CFD
from __future__ import unicode_literals
import numpy


T = ['CGNSTree', None, [
    ['CGNSLibraryVersion', numpy.array([3.200000047683716], dtype='float32', order='C'), [], 'CGNSLibraryVersion_t'],
    ['{Base#1}', numpy.array([3, 3], dtype='int32', order='C'), [
        ['Interior', None, [
            ['FamilyBC', numpy.array(['B', 'C', 'F', 'a', 'r', 'f', 'i', 'e', 'l', 'd'], dtype='S', order='C'), [],
             'FamilyBC_t'], ], 'Family_t'],
        ['Exterior', None, [
            ['FamilyBC', numpy.array(['B', 'C', 'W', 'a', 'l', 'l'], dtype='S', order='C'), [], 'FamilyBC_t'], ],
         'Family_t'],
        ['{Zone-A}', numpy.array([[13, 12, 0], [61, 60, 0], [17, 16, 0]], dtype='int64', order='F'), [
            ['ZoneType', numpy.array(['S', 't', 'r', 'u', 'c', 't', 'u', 'r', 'e', 'd'], dtype='S', order='C'), [],
             'ZoneType_t'],
            ['GridCoordinates', None, [
                ['CoordinateX', None, [], 'DataArray_t'],
                ['CoordinateY', None, [], 'DataArray_t'],
                ['CoordinateZ', None, [], 'DataArray_t'], ], 'GridCoordinates_t'],
            ['ZoneBC', None, [
                ['{BC-2}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 13], [1, 1], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['I', 'n', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-1}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 13], [1, 61], [1, 1]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-4}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 13], [61, 61], [1, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-6}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 13], [1, 61], [17, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'], ], 'ZoneBC_t'],
            ['ZoneGridConnectivity', None, [
                ['{CT-A-D}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'D', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['A', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['D', '_', 'A'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[1, 1], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['A', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['D', '_', 'A'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-A-B}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'B', '}'], dtype='S', order='C'), [
                    ['Transform', numpy.array([1, 2, 3], dtype='int32', order='C'), [], '"int[IndexDimension]"'],
                    ['PointRange', numpy.array([[13, 13], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['PointRangeDonor', numpy.array([[1, 1], [1, 61], [1, 17]], dtype='int32', order='F'), [],
                     'IndexRange_t'], ], 'GridConnectivity1to1_t'], ], 'ZoneGridConnectivity_t'],
            ['FlowSolution#EndOfRun', None, [
                ['GridLocation', numpy.array(['C', 'e', 'l', 'l', 'C', 'e', 'n', 't', 'e', 'r'], dtype='S', order='C'),
                 [], 'GridLocation_t'],
                ['Density', None, [], 'DataArray_t'],
                ['MomentumX', None, [], 'DataArray_t'],
                ['MomentumY', None, [], 'DataArray_t'],
                ['MomentumZ', None, [], 'DataArray_t'],
                ['EnergyStagnationDensity', None, [], 'DataArray_t'], ], 'FlowSolution_t'], ], 'Zone_t'],
        ['{Zone-B}', numpy.array([[31, 30, 0], [61, 60, 0], [17, 16, 0]], dtype='int64', order='F'), [
            ['ZoneType', numpy.array(['S', 't', 'r', 'u', 'c', 't', 'u', 'r', 'e', 'd'], dtype='S', order='C'), [],
             'ZoneType_t'],
            ['GridCoordinates', None, [
                ['CoordinateX', None, [], 'DataArray_t'],
                ['CoordinateY', None, [], 'DataArray_t'],
                ['CoordinateZ', None, [], 'DataArray_t'], ], 'GridCoordinates_t'],
            ['ZoneBC', None, [
                ['{BC-2}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 31], [1, 1], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['I', 'n', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-1}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 31], [1, 61], [1, 1]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-4}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 31], [61, 61], [1, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-6}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 31], [1, 61], [17, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'], ], 'ZoneBC_t'],
            ['ZoneGridConnectivity', None, [
                ['{CT-B-A}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'A', '}'], dtype='S', order='C'), [
                    ['Transform', numpy.array([1, 2, 3], dtype='int32', order='C'), [], '"int[IndexDimension]"'],
                    ['PointRange', numpy.array([[1, 1], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['PointRangeDonor', numpy.array([[13, 13], [1, 61], [1, 17]], dtype='int32', order='F'), [],
                     'IndexRange_t'], ], 'GridConnectivity1to1_t'],
                ['{CT-B-C}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'C', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['B', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['C', '_', 'B'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[31, 31], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['B', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['C', '_', 'B'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'], ], 'ZoneGridConnectivity_t'],
            ['FlowSolution#EndOfRun', None, [
                ['GridLocation', numpy.array(['C', 'e', 'l', 'l', 'C', 'e', 'n', 't', 'e', 'r'], dtype='S', order='C'),
                 [], 'GridLocation_t'],
                ['Density', None, [], 'DataArray_t'],
                ['MomentumX', None, [], 'DataArray_t'],
                ['MomentumY', None, [], 'DataArray_t'],
                ['MomentumZ', None, [], 'DataArray_t'],
                ['EnergyStagnationDensity', None, [], 'DataArray_t'], ], 'FlowSolution_t'], ], 'Zone_t'],
        ['{Zone-C1}', numpy.array([[27, 26, 0], [25, 24, 0], [17, 16, 0]], dtype='int64', order='F'), [
            ['ZoneType', numpy.array(['S', 't', 'r', 'u', 'c', 't', 'u', 'r', 'e', 'd'], dtype='S', order='C'), [],
             'ZoneType_t'],
            ['GridCoordinates', None, [
                ['CoordinateX', None, [], 'DataArray_t'],
                ['CoordinateY', None, [], 'DataArray_t'],
                ['CoordinateZ', None, [], 'DataArray_t'], ], 'GridCoordinates_t'],
            ['ZoneBC', None, [
                ['{BC-1}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 27], [1, 25], [1, 1]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-4}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 27], [25, 25], [1, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-6}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 27], [1, 25], [17, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'], ], 'ZoneBC_t'],
            ['ZoneGridConnectivity', None, [
                ['{CT-C1-C2}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'C', '2', '}'], dtype='S', order='C'), [
                    ['Transform', numpy.array([1, 2, 3], dtype='int32', order='C'), [], '"int[IndexDimension]"'],
                    ['PointRange', numpy.array([[1, 27], [1, 1], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['PointRangeDonor', numpy.array([[1, 27], [37, 37], [1, 17]], dtype='int32', order='F'), [],
                     'IndexRange_t'], ], 'GridConnectivity1to1_t'],
                ['{CT-C1-B}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'B', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['C', '_', 'B'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['B', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[1, 1], [1, 25], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['C', '_', 'B'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['B', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-C1-D}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'D', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[27, 27], [1, 25], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'], ], 'ZoneGridConnectivity_t'],
            ['FlowSolution#EndOfRun', None, [
                ['GridLocation', numpy.array(['C', 'e', 'l', 'l', 'C', 'e', 'n', 't', 'e', 'r'], dtype='S', order='C'),
                 [], 'GridLocation_t'],
                ['Density', None, [], 'DataArray_t'],
                ['MomentumX', None, [], 'DataArray_t'],
                ['MomentumY', None, [], 'DataArray_t'],
                ['MomentumZ', None, [], 'DataArray_t'],
                ['EnergyStagnationDensity', None, [], 'DataArray_t'], ], 'FlowSolution_t'], ], 'Zone_t'],
        ['{Zone-C2}', numpy.array([[27, 26, 0], [37, 36, 0], [17, 16, 0]], dtype='int64', order='F'), [
            ['ZoneType', numpy.array(['S', 't', 'r', 'u', 'c', 't', 'u', 'r', 'e', 'd'], dtype='S', order='C'), [],
             'ZoneType_t'],
            ['GridCoordinates', None, [
                ['CoordinateX', None, [], 'DataArray_t'],
                ['CoordinateY', None, [], 'DataArray_t'],
                ['CoordinateZ', None, [], 'DataArray_t'], ], 'GridCoordinates_t'],
            ['ZoneBC', None, [
                ['{BC-2}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 27], [1, 1], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['I', 'n', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-1}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 27], [1, 37], [1, 1]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-6}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 27], [1, 37], [17, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'], ], 'ZoneBC_t'],
            ['ZoneGridConnectivity', None, [
                ['{CT-C2-B}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'B', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['C', '_', 'B'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['B', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[1, 1], [1, 37], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['C', '_', 'B'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['B', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-C2-D}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'D', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[27, 27], [1, 37], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-C2-C1}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'C', '1', '}'], dtype='S', order='C'), [
                    ['Transform', numpy.array([1, 2, 3], dtype='int32', order='C'), [], '"int[IndexDimension]"'],
                    ['PointRange', numpy.array([[1, 27], [37, 37], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['PointRangeDonor', numpy.array([[1, 27], [1, 1], [1, 17]], dtype='int32', order='F'), [],
                     'IndexRange_t'], ], 'GridConnectivity1to1_t'], ], 'ZoneGridConnectivity_t'],
            ['FlowSolution#EndOfRun', None, [
                ['GridLocation', numpy.array(['C', 'e', 'l', 'l', 'C', 'e', 'n', 't', 'e', 'r'], dtype='S', order='C'),
                 [], 'GridLocation_t'],
                ['Density', None, [], 'DataArray_t'],
                ['MomentumX', None, [], 'DataArray_t'],
                ['MomentumY', None, [], 'DataArray_t'],
                ['MomentumZ', None, [], 'DataArray_t'],
                ['EnergyStagnationDensity', None, [], 'DataArray_t'], ], 'FlowSolution_t'], ], 'Zone_t'],
        ['{Zone-D1}', numpy.array([[41, 40, 0], [61, 60, 0], [17, 16, 0]], dtype='int64', order='F'), [
            ['ZoneType', numpy.array(['S', 't', 'r', 'u', 'c', 't', 'u', 'r', 'e', 'd'], dtype='S', order='C'), [],
             'ZoneType_t'],
            ['GridCoordinates', None, [
                ['CoordinateX', None, [], 'DataArray_t'],
                ['CoordinateY', None, [], 'DataArray_t'],
                ['CoordinateZ', None, [], 'DataArray_t'], ], 'GridCoordinates_t'],
            ['ZoneBC', None, [
                ['{BC-2}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 41], [1, 1], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['I', 'n', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-4}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 41], [61, 61], [1, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-6}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 41], [1, 61], [17, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'], ], 'ZoneBC_t'],
            ['ZoneGridConnectivity', None, [
                ['{CT-D1-C}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'C', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[1, 1], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-D1-A}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'A', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['D', '_', 'A'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['A', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[41, 41], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['D', '_', 'A'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['A', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-D1-D2}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'D', '2', '}'], dtype='S', order='C'), [
                    ['Transform', numpy.array([1, 2, 3], dtype='int32', order='C'), [], '"int[IndexDimension]"'],
                    ['PointRange', numpy.array([[1, 41], [1, 61], [1, 1]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['PointRangeDonor', numpy.array([[1, 41], [1, 61], [17, 17]], dtype='int32', order='F'), [],
                     'IndexRange_t'], ], 'GridConnectivity1to1_t'], ], 'ZoneGridConnectivity_t'],
            ['FlowSolution#EndOfRun', None, [
                ['GridLocation', numpy.array(['C', 'e', 'l', 'l', 'C', 'e', 'n', 't', 'e', 'r'], dtype='S', order='C'),
                 [], 'GridLocation_t'],
                ['Density', None, [], 'DataArray_t'],
                ['MomentumX', None, [], 'DataArray_t'],
                ['MomentumY', None, [], 'DataArray_t'],
                ['MomentumZ', None, [], 'DataArray_t'],
                ['EnergyStagnationDensity', None, [], 'DataArray_t'], ], 'FlowSolution_t'], ], 'Zone_t'],
        ['{Zone-D2}', numpy.array([[41, 40, 0], [61, 60, 0], [17, 16, 0]], dtype='int64', order='F'), [
            ['ZoneType', numpy.array(['S', 't', 'r', 'u', 'c', 't', 'u', 'r', 'e', 'd'], dtype='S', order='C'), [],
             'ZoneType_t'],
            ['GridCoordinates', None, [
                ['CoordinateX', None, [], 'DataArray_t'],
                ['CoordinateY', None, [], 'DataArray_t'],
                ['CoordinateZ', None, [], 'DataArray_t'], ], 'GridCoordinates_t'],
            ['ZoneBC', None, [
                ['{BC-2}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 41], [1, 1], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['I', 'n', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-1}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 41], [1, 61], [1, 1]], dtype='int32', order='F'), [], 'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'],
                ['{BC-4}',
                 numpy.array(['F', 'a', 'm', 'i', 'l', 'y', 'S', 'p', 'e', 'c', 'i', 'f', 'i', 'e', 'd'], dtype='S',
                             order='C'), [
                     ['PointRange', numpy.array([[1, 41], [61, 61], [1, 17]], dtype='int32', order='F'), [],
                      'IndexRange_t'],
                     ['FamilyName', numpy.array(['E', 'x', 't', 'e', 'r', 'i', 'o', 'r'], dtype='S', order='C'), [],
                      'FamilyName_t'], ], 'BC_t'], ], 'ZoneBC_t'],
            ['ZoneGridConnectivity', None, [
                ['{CT-D2-C}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'C', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[1, 1], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['D', '_', 'C'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['C', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'],
                ['{CT-D2-D1}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'D', '1', '}'], dtype='S', order='C'), [
                    ['Transform', numpy.array([1, 2, 3], dtype='int32', order='C'), [], '"int[IndexDimension]"'],
                    ['PointRange', numpy.array([[1, 41], [1, 61], [17, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['PointRangeDonor', numpy.array([[1, 41], [1, 61], [1, 1]], dtype='int32', order='F'), [],
                     'IndexRange_t'], ], 'GridConnectivity1to1_t'],
                ['{CT-D2-A}', numpy.array(['{', 'Z', 'o', 'n', 'e', '-', 'A', '}'], dtype='S', order='C'), [
                    ['GridConnectivityType',
                     numpy.array(['A', 'b', 'u', 't', 't', 'i', 'n', 'g'], dtype='S', order='C'), [],
                     'GridConnectivityType_t'],
                    ['ConnectivityKeyLocal', numpy.array(['D', '_', 'A'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['ConnectivityKeyDonor', numpy.array(['A', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                    ['PointRange', numpy.array([[41, 41], [1, 61], [1, 17]], dtype='int32', order='F'), [], 'IndexRange_t'],
                    ['.Solver#Property', None, [
                        ['globborder', numpy.array(['D', '_', 'A'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['globborderdonor', numpy.array(['A', '_', 'D'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['jtype', numpy.array(['n', 'o', 'm', 'a', 't', 'c', 'h'], dtype='S', order='C'), [],
                         'DataArray_t'],
                        ['nomatch_special', numpy.array(['n', 'o', 'n', 'e'], dtype='S', order='C'), [], 'DataArray_t'],
                        ['type', numpy.array(['j', 'o', 'i', 'n'], dtype='S', order='C'), [], 'DataArray_t'], ],
                     'UserDefinedData_t'], ], 'GridConnectivity_t'], ], 'ZoneGridConnectivity_t'],
            ['FlowSolution#EndOfRun', None, [
                ['GridLocation', numpy.array(['C', 'e', 'l', 'l', 'C', 'e', 'n', 't', 'e', 'r'], dtype='S', order='C'),
                 [], 'GridLocation_t'],
                ['Density', None, [], 'DataArray_t'],
                ['MomentumX', None, [], 'DataArray_t'],
                ['MomentumY', None, [], 'DataArray_t'],
                ['MomentumZ', None, [], 'DataArray_t'],
                ['EnergyStagnationDensity', None, [], 'DataArray_t'], ], 'FlowSolution_t'], ], 'Zone_t'],
        ['ReferenceState', None, [
            ['ReferenceStateDescription', numpy.array(
                ['G', 'l', 'o', 'b', 'a', 'l', ' ', 'r', 'e', 'f', 'e', 'r', 'e', 'n', 'c', 'e', ' ', 's', 't', 'a',
                 't', 'e'], dtype='S', order='C'), [], 'Descriptor_t'],
            ['AngleofAttack', numpy.array([7.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['BetaAngle', numpy.array([0.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Coef_Area', numpy.array([0.25], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Coef_PressureDynamic', numpy.array([2837.15315566], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Density', numpy.array([1.22524863848], dtype='float64', order='C'), [], 'DataArray_t'],
            ['EnergyStagnationDensity', numpy.array([256154.399197], dtype='float64', order='C'), [], 'DataArray_t'],
            ['LengthReference', numpy.array([0.5], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Mach', numpy.array([0.2], dtype='float64', order='C'), [], 'DataArray_t'],
            ['MomentumX', numpy.array([82.7597580354], dtype='float64', order='C'), [], 'DataArray_t'],
            ['MomentumY', numpy.array([0.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['MomentumZ', numpy.array([10.1616205508], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Pressure', numpy.array([101326.898416], dtype='float64', order='C'), [], 'DataArray_t'],
            ['PressureStagnation', numpy.array([104192.536703], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Reynolds', numpy.array([2330000.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['Temperature', numpy.array([288.15], dtype='float64', order='C'), [], 'DataArray_t'],
            ['TemperatureStagnation', numpy.array([290.4552], dtype='float64', order='C'), [], 'DataArray_t'],
            ['TurbulentSANuTilde', numpy.array([1.46035471482e-08], dtype='float64', order='C'), [], 'DataArray_t'],
            ['VelocityMagnitude', numpy.array([68.0525297105], dtype='float64', order='C'), [], 'DataArray_t'], ],
         'ReferenceState_t'],
        ['.Solver#Compute', None, [
            ['artviscosity', numpy.array(['d', 'i', 's', 's', 'c', 'a'], dtype='S', order='C'), [], 'DataArray_t'],
            ['avcoef_k2', numpy.array([1.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['avcoef_k4', numpy.array([0.032], dtype='float64', order='C'), [], 'DataArray_t'],
            ['avcoef_sigma', numpy.array([1.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['cfl', numpy.array([1.0], dtype='float64', order='C'), [], 'DataArray_t'],
            ['fluid', numpy.array(['I', 'd', 'e', 'a', 'l'], dtype='S', order='C'), [], 'DataArray_t'],
            ['flux', numpy.array(['j', 'a', 'm', 'e', 's', 'o', 'n'], dtype='S', order='C'), [], 'DataArray_t'],
            ['niter', numpy.array([1000], dtype='int32', order='C'), [], 'DataArray_t'],
            ['ode', numpy.array(['r', 'k', '4'], dtype='S', order='C'), [], 'DataArray_t'],
            ['phymod', numpy.array(['N', 'S', 'T', 'u', 'r', 'b', 'u', 'l', 'e', 'n', 't'], dtype='S', order='C'), [],
             'DataArray_t'],
            ['t_cutvar1', numpy.array([1.78929762604e-08], dtype='float64', order='C'), [], 'DataArray_t'],
            ['t_harten', numpy.array([1e-24], dtype='float64', order='C'), [], 'DataArray_t'],
            ['time_algo', numpy.array(['s', 't', 'e', 'a', 'd', 'y'], dtype='S', order='C'), [], 'DataArray_t'],
            ['turbmod', numpy.array([10], dtype='int32', order='C'), [], 'DataArray_t'],
            ['visclaw', numpy.array(['S', 'u', 't', 'h', 'e', 'r', 'l', 'a', 'n', 'd'], dtype='S', order='C'), [],
             'DataArray_t'],
            ['walldistcompute',
             numpy.array(['m', 'i', 'n', 'i', 'i', 'n', 't', 'e', 'r', 'f', '_', 'o', 'r', 't', 'h', 'o'], dtype='S',
                         order='C'), [], 'DataArray_t'], ], 'UserDefinedData_t'], ], 'CGNSBase_t'], ], 'CGNSTree_t']



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import print_function
import os
import sys
import subprocess

import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import CGNS.MAP as CGM
import CGNS.version


class Context:
    def __init__(self):
        self.converter = None


def openFile(filename):
    flags = CGM.S2P_DEFAULTS | CGM.S2P_NODATA
    (t, l, p) = CGM.load(filename, flags=flags, maxdata=33, lksearch=['.'])
    return (t, l, p, filename)


def parseFile(filename, P, C, L):
    C.depth += 1
    R = []
    T = openFile(filename)
    LK = T[1]
    if (C.translate):
        LK = transLinks(filename, LK, P, C)
    searchLinks(LK, C, R)
    for p in R:
        L.append((filename, p))
        if (C.path):
            P.append('%s' % (p[3],))
        else:
            P.append('%s:%s' % (T[3], p[3]))
    for l in LK:
        if (l[0] == ''):
            FH = l[1]
        else:
            FH = "%s/%s" % (l[0], l[1])
        parseFile(FH, P, C, L)
    C.depth -= 1
    return P, L


def linkErrorAsString(code):
    s = ""
    if (code & CGM.S2P_LKOK):
        s += 'Link ok'
    if (code & CGM.S2P_LKFAIL):
        s += 'Link Failed:'
    if (code & CGM.S2P_LKBADSYNTAX):
        s += 'Link bad syntax'
    if (code & CGM.S2P_LKNOFILE):
        s += 'Linked-to file not found'
    if (code & CGM.S2P_LKFILENOREAD):
        s += 'Linked-to file not readable'
    if (code & CGM.S2P_LKNONODE):
        s += 'Linked-to node not found in file'
    if (code & CGM.S2P_LKLOOP):
        s += 'Link loop detected'
    if (code & CGM.S2P_LKIGNORED):
        s += 'Link ignored'
    return s


def checkString(variable, targetlist, re):
    if (variable is None):
        return False
    if (not re):
        return (variable in targetlist)
    else:
        for t in targetlist:
            if (variable.search(t) is not None):
                return True
        return False


def searchLinks(L, C, R):
    for l in L:
        add = True
        if (add or C.linklist):
            R.append(l)


def asHDFname(FA, C):
    return os.path.splitext(FA)[0] + C.exthdf


def convertInPlace(FA, FH, C):
    if not os.path.isfile(FA):
        if C.verbose:
            print('   ' * C.depth + " Error: Unreachable file: %s" % FA)
        return False
    elif not CGM.probe(FA):
        subprocess.check_output([C.converter, "-h", FA, FH])
        return True
    else:
        if C.verbose:
            print('   ' * C.depth + " Error: Mixing links to ADF and HDF files...")
        return False


def transLinks(filename, L, P, C):
    LH = []
    for l in L:
        LN = asHDFname(l[1], C)
        if (l[0] == ''):
            FA = l[1]
        else:
            FA = "%s/%s" % (l[0], l[1])
        if (C.verbose):
            print('   ' * C.depth, '->', FA)
        FH = asHDFname(FA, C)
        if (convertInPlace(FA, FH, C)):
            LH.append([l[0], LN, l[2], l[3]])
    (t, l, p) = CGM.load(filename)
    CGM.save(filename, t, links=LH)
    return LH

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import string

CHECK_NONE = 0
CHECK_OK = CHECK_GOOD = CHECK_PASS = 1
CHECK_INFO = CHECK_WARN = 2
CHECK_BAD = CHECK_ERROR = CHECK_FAIL = 3
CHECK_USER = 4


# def sortDiagByKey(d1, d2):
#     (k1, k2) = (d1.key, d2.key)
#     return k1 > k2


def getWorst(st1, st2):
    if (CHECK_FAIL in [st1, st2]):
        return CHECK_FAIL
    if (CHECK_USER in [st1, st2]):
        return CHECK_USER
    if (CHECK_WARN in [st1, st2]):
        return CHECK_WARN
    if (CHECK_GOOD in [st1, st2]):
        return CHECK_GOOD
    return CHECK_NONE


class DiagnosticMessagePattern(object):
    __levelstr = {CHECK_FAIL: 'E', CHECK_USER: 'U',
                  CHECK_WARN: 'W', CHECK_GOOD: ' ', CHECK_NONE: '?'}

    def __init__(self, mkey, mlevel, mstring):
        self._key = mkey
        self._lvl = mlevel
        self._str = mstring

    @property
    def key(self):
        return self._key

    @property
    def level(self):
        return self._lvl

    @property
    def message(self):
        return self._str

    def levelAsStr(self):
        return self.__levelstr[self._lvl]

    def __str__(self):
        return '[%s:%s] %s' % (self._key, self.levelAsStr(), self._str)

    def notSubst(self):
        return '%' in self._str

    def forceLevel(self, l):
        self._lvl = l


class DiagnosticMessageInstance(DiagnosticMessagePattern):
    def __init__(self, pattern):
        super(DiagnosticMessageInstance, self).__init__(pattern.key, pattern.level, pattern.message)

    def substitute(self, *tp):
        msg = self._str
        try:
            if (tp): msg = msg % tp
        except TypeError:
            pass
        self._str = msg
        return self

    def __str__(self):
        return '("%s","%s","""%s""")' % (self._key, self.levelAsStr(), self._str)


class DiagnosticLog(dict):
    __messages = {}

    def __init__(self):
        dict.__init__(self)

    def merge(self, log):
        self.update(log)

    def listMessages(self):
        return self.__messages

    def noContextMessage(self, m):
        if (DiagnosticLog.__messages[m].notSubst()): return None
        return DiagnosticLog.__messages[m].message

    def addMessage(self, k, m):
        DiagnosticLog.__messages[k] = DiagnosticMessageInstance(*m)

    def addMessages(self, d):
        for e in d:
            DiagnosticLog.__messages[e] = DiagnosticMessagePattern(e, d[e][0], d[e][1])

    def push(self, path, messagekey, *tp):
        if (path is None): return
        if (path not in self): self[path] = []
        if (messagekey not in DiagnosticLog.__messages): return
        entry = DiagnosticMessageInstance(DiagnosticLog.__messages[messagekey])
        self[path].append(entry.substitute(*tp))
        return DiagnosticLog.__messages[messagekey].level

    def __len__(self):
        return len(self.keys())

    def shift(self, path, shiftstring=' '):
        n = path.split('/')
        return len(n) * shiftstring

    def getWorst(self, st1, st2):
        return getWorst(st1, st2)

    def status(self, entry):
        return entry.level

    def hasOnlyKey(self, path, keylist):
        k1 = set(keylist)
        k2 = set([e.key for e in self[path]])
        return k2.issubset(k1)

    def key(self, entry):
        return entry.key

    def message(self, entry, path=None):
        shft = ''
        if (path is not None): shft = self.shift(path)
        s = '%s[%s:%s] %s' % (shft, entry.key, entry.levelAsStr(), entry.message)
        return s

    def getWorstDiag(self, path):
        s = set([e.level for e in self[path]])
        try:
            r = list(s)[0]
            for x in s:
                r = getWorst(r, x)
        except:
            r = CHECK_NONE
        return r

    def diagForPath(self, path):
        if (path in self): return self[path]
        return None

    def allMessageKeys(self):
        r = set()
        for path in self:
            for entry in self[path]:
                r.add(entry.key)
        mlist = list(r)
        mlist.sort()
        return mlist

    def allPathKeys(self):
        return list(self)

    def diagnosticsByPath(self, path):
        if (self.diagForPath(path) is not None):
            dlist = self[path]
            dlist.sort(key=lambda x: x.key)
            for diag in dlist:
                yield (diag, path)

    def diagnosticsByMessage(self, msg):
        plist = list(self)
        plist.sort()
        for path in plist:
            for diag in self[path]:
                if (diag.key == msg): yield (diag, path)

    def __str__(self):
        s = "{\n"
        for path in self:
            s += "'%s':\n" % path
            for diag in self[path]:
                s += "  %s,\n" % diag
        s += "}\n"
        return s

    def forceAsWarning(self, key):
        if (key in DiagnosticLog.__messages):
            DiagnosticLog.__messages[key].forceLevel(CHECK_WARN)

    def forceAsFailure(self, key):
        if (key in DiagnosticLog.__messages):
            DiagnosticLog.__messages[key].forceLevel(CHECK_FAIL)

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

colordict = {'high': '#7ebaff', 'low': '#78b5e9', 'high2': '#bad5f4', 'low2': '#bad5ff'}

Q7CONTROLVIEWSTYLESHEET = """
 QTableView {
 show-decoration-selected: 1;
 }

 QTableView::item:selected {
 background:qlineargradient(x1:0,y1:0,x2:0,y2:1,stop:0 %(high)s,stop:1 %(low)s);
 color:black;
 border: 1px solid #bfcde4;
 }

 QTableView::item
 {
 border: 0px;
 padding: 0px;
 }
""" % colordict

Q7TABLEVIEWSTYLESHEET = """
 Q7TableView {
 show-decoration-selected: 1;
 }

 Q7TableView::item:selected
 {
  border: 1px solid %(high)s;
  font: bold;
 }

 Q7TableView::item
 {
  border: 0px;
  padding: 0px;
  font: fixed 8;
 }
""" % colordict

Q7TREEVIEWSTYLESHEET = """
 QTreeView {
 show-decoration-selected: 1;
 }

 QTreeView::branch {
 background: palette(base);
 }

 QTreeView::item [diffNA="true"] { background-color: yellow }

 QTreeView::item {
 border: 1px solid #d9d9d9;
 border-top-color: transparent;
 border-bottom-color: transparent;
 text-align:right;
 color:black;
 }

 QTreeView::item:selected {
 border: 1px solid #567dbc;
 }

 QTreeView::item:selected:active{
 background:qlineargradient(x1:0,y1:0,x2:0,y2:1,stop:0 %(high)s,stop:1 %(low)s);
 }

 QTreeView::item:selected:!active {
 background:qlineargradient(x1:0,y1:0,x2:0,y2:1,stop:0 %(high)s,stop:1 %(low)s);
 }

 QTreeView::branch:has-siblings:!adjoins-item {
 border-image: url(:/images/icons/vline.png) 0;
 }

 QTreeView::branch:has-siblings:adjoins-item {
 border-image: url(:/images/icons/branch-more.png) 0;
 }

 QTreeView::branch:!has-children:!has-siblings:adjoins-item {
 border-image: url(:/images/icons/branch-end.png) 0;
 }

 QTreeView::branch:closed:has-children:has-siblings {
 border-image: url(:/images/icons/branch-more-p.png) 0;
 }

 QTreeView::branch:has-children:!has-siblings:closed {
 border-image: url(:/images/icons/branch-end-p.png) 0;
 }

 QTreeView::branch:open:has-children:has-siblings {
 background: magenta;
 }

 QTreeView::branch:open:has-children:!has-siblings {
 border-image: url(:/images/icons/branch-end.png) 0;
 }

""" % colordict

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
pass
# run the whole suite with:
#
#   python -c 'import CGNS.VAL.suite.run'
#
# To save all tests files in a CGNS/HDF files, set the shell variable
# CGNS_VAL_SAVE_FILES, create a directory and run the test in this directory.
#
#   mkdir files
#   cd files
#   export CGNS_VAL_SAVE_FILES=1
#   python -c 'import CGNS.VAL.suite.run'
#
# To run and compare with cgnscheck:
#
#   export CGNS_VAL_RUN_CGNSCHECK=1
#
# Files *.hdf are results or the CGNS/Python tree creation, *-fail-*
# trees may be unreadable.
# Files *.diag are CGNS.VAL output diagnostic
# Files *.cgnscheck are output files from cgnscheck
#
# Do not forget to remove files from one run to another...
#



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

data = C.newBase(None, '{Base}', 3, 3)
C.newZone(data, '{Zone}', N.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
C.newSimulationType(data)
C.newIntegralData(data, '{IntegralData}')
C.newBaseIterativeData(data, '{BaseIterativeData}')
C.newConvergenceHistory(data)
C.newFamily(data, '{Family}')
C.newFlowEquationSet(data)
C.newReferenceState(data)
C.newAxisymmetry(data)
C.newRotatingCoordinates(data)
C.newGravity(data)
C.newDataClass(data)
C.newDimensionalUnits(data)
C.newUserDefinedData(data, '{UserDefinedData}')
C.newDescriptor(data, '{Descriptor}')

status = '6.2'
comment = 'Full SIDS with all optionals children'
pattern = [data, status, comment]



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from __future__ import print_function
from builtins import (str, bytes, range, dict)

import sys
import time

from CGNS.pyCGNSconfig import HAS_MSW

if HAS_MSW:
    import ctypes
    myappid = u'pycgns.pycgns.cglook.version' # arbitrary string
    ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

from CGNS.NAV.moption import Q7OptionContext as OCTXT

from qtpy import QtCore
from qtpy.QtWidgets import QApplication, QSplashScreen
from qtpy.QtGui import QPixmap
#from qtpy.QtGui import *
from CGNS.NAV.wcontrol import Q7Main

splash = None
wcontrol = None

def closeSplash():
    global splash
    global wcontrol
    splash.finish(wcontrol)
    
# -----------------------------------------------------------------
def run(args=[], files=[], datasets=[],
        flags=(False, False, False, False, False, False, False),
        ppath=None, query=None):
    hidecontrol = False
    global splash
    global wcontrol
    
    if flags[4]:
        from CGNS.NAV.wquery import Q7Query
        Q7Query.loadUserQueries()
        Q7Query.fillQueries()
        ql = Q7Query.queriesNamesList()
        print('# CGNS.NAV available queries:')
        for q in ql:
            print(' ', q)
    else:
        app = QApplication(args)
        if not flags[5]:
            pixmap = QPixmap(":/images/splash.png")
            splash = QSplashScreen(pixmap, QtCore.Qt.WindowStaysOnTopHint)
            splash.show()
            splash.showMessage("Release v%s" % OCTXT._ToolVersion,
                               QtCore.Qt.AlignHCenter | QtCore.Qt.AlignBottom)
            timer = QtCore.QTimer.singleShot(3000,closeSplash)
        app.processEvents()
        Q7Main.verbose = flags[2]
        wcontrol = Q7Main()
        wcontrol._application = app
        wcontrol.setOptionValue('NAVTrace', flags[2])
        wcontrol.transientRecurse = flags[0]
        wcontrol.transientVTK = flags[3]
        wcontrol.query = query
        wcontrol._T('start')
        wcontrol.setDefaults()
        if flags[1]:
            wcontrol.loadlast()
            hidecontrol = flags[6]
        if files:
            for ff in files:
                wcontrol.loadfile(ff)
            hidecontrol = flags[6]
        if datasets:
            for dd in datasets:
                wcontrol.loadCompleted(dataset_name='FFFF',
                                       dataset_base='BASE',
                                       dataset_tree=dd,
                                       dataset_references=[],
                                       dataset_paths=[])
            hidecontrol = flags[6]
        wcontrol.show()
        if hidecontrol:
            wcontrol.hide()
        app.exec_()
        wcontrol._T('leave')
    sys.exit()


# run()

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'base dataclass #1'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newDataClass(b, CGK.NondimensionalParameter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'base dataclass #2'
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newDataClass(b, CGK.NormalizedByDimensional_s)
d[0] = 'dataclass'
d = CGL.newDataClass(b, CGK.NormalizedByDimensional_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'base dataclass #3'
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newDataClass(b, CGK.NormalizedByDimensional_s)
d[1] = CGU.setStringAsArray("NormalizedByDimensionnal")
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'all levels dataclass'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newFamily(b, '{Family}')
d = CGL.newDataClass(b, CGK.NondimensionalParameter_s)
z = CGL.newZone(b, '{Zone}', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
d = CGL.newDataClass(z, CGK.NondimensionalParameter_s)
g = CGL.newGridCoordinates(z, CGK.GridCoordinates_s)
w = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
d = CGL.newDataClass(g, CGK.NondimensionalParameter_s)
f = CGL.newFlowSolution(z)
d = CGL.newDataClass(f, CGK.NondimensionalParameter_s)
a = CGL.newDataArray(f, '{DataArray}', value=NPY.ones((4, 6, 8), dtype='float64', order='F'))
d = CGL.newDataClass(a, CGK.NondimensionalParameter_s)
n = CGL.newZoneBC(z)
d = CGL.newDataClass(n, CGK.NondimensionalParameter_s)
q = CGL.newBC(n, '{BC}', family='{Family}')
d = CGL.newDataClass(q, CGK.NondimensionalParameter_s)
s = CGL.newBCDataSet(q, '{Set#01}')
d = CGL.newDataClass(s, CGK.NondimensionalParameter_s)
c = CGL.newBCData(s, CGK.Dirichlet_s)
d = CGL.newDataClass(c, CGK.NondimensionalParameter_s)
r = CGL.newReferenceState(b)
d = CGL.newDataClass(r, CGK.NondimensionalParameter_s)
a = CGL.newAxisymmetry(b)
d = CGL.newDataClass(a, CGK.NondimensionalParameter_s)
a = CGL.newRotatingCoordinates(b)
d = CGL.newDataClass(a, CGK.NondimensionalParameter_s)
a = CGL.newDiscreteData(z, '{DiscreteData}')
d = CGL.newDataClass(a, CGK.NondimensionalParameter_s)
i = CGL.newBaseIterativeData(b, '{BaseIterativeData}')
d = CGL.newDataClass(i, CGK.NondimensionalParameter_s)
i = CGL.newZoneIterativeData(z, '{ZoneIterativeData}')
d = CGL.newDataClass(i, CGK.NondimensionalParameter_s)
m = CGL.newRigidGridMotion(z, '{RigidGridMotion}',
                           vector=NPY.array([[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]))
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newArbitraryGridMotion(z, '{ArbitraryGridMotion}')
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
x = CGL.newZoneGridConnectivity(z)
x = CGL.newGridConnectivity(x, '{GridConnectivity}', z[0])
CGL.newPointRange(x, value=NPY.array([[1, 1], [1, 1], [1, 1]], dtype=NPY.int32, order='F'))
p = CGL.newGridConnectivityProperty(x)
m = CGL.newPeriodic(p)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
w = CGL.newConvergenceHistory(b)
d = CGL.newDataClass(w, CGK.NondimensionalParameter_s)
i = CGL.newIntegralData(b, '{IntegralData}')
d = CGL.newDataClass(i, CGK.NondimensionalParameter_s)
i = CGL.newUserDefinedData(b, '{UserDefinedData}')
d = CGL.newDataClass(i, CGK.NondimensionalParameter_s)
i = CGL.newGravity(b)
d = CGL.newDataClass(i, CGK.NondimensionalParameter_s)
f = CGL.newFlowEquationSet(b)
d = CGL.newDataClass(f, CGK.NondimensionalParameter_s)
m = CGL.newGasModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newViscosityModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newThermalConductivityModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newThermalRelaxationModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newChemicalKineticsModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newTurbulenceClosure(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newTurbulenceModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newEMConductivityModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newEMMagneticFieldModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
m = CGL.newEMElectricFieldModel(f)
d = CGL.newDataClass(m, CGK.NondimensionalParameter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'all levels dataclass/dimensionalunits'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newFamily(b, '{Family}')
d = CGL.newDataClass(b, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(b)
z = CGL.newZone(b, '{Zone}', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
d = CGL.newDataClass(z, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(z)
g = CGL.newGridCoordinates(z, CGK.GridCoordinates_s)
w = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
d = CGL.newDataClass(g, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(g)
f = CGL.newFlowSolution(z)
d = CGL.newDataClass(f, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(f)
a = CGL.newDataArray(f, '{DataArray}', value=NPY.ones((4, 6, 8), dtype='float64', order='F'))
d = CGL.newDataClass(a, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(a)
n = CGL.newZoneBC(z)
d = CGL.newDataClass(n, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(n)
q = CGL.newBC(n, '{BC}', family='{Family}')
d = CGL.newDataClass(q, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(q)
s = CGL.newBCDataSet(q, '{BCDataSet}')
d = CGL.newDataClass(s, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(s)
c = CGL.newBCData(s, CGK.Neumann_s)
d = CGL.newDataClass(c, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(c)
r = CGL.newReferenceState(b)
d = CGL.newDataClass(r, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(r)
a = CGL.newAxisymmetry(b)
d = CGL.newDataClass(a, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(a)
a = CGL.newRotatingCoordinates(b)
d = CGL.newDataClass(a, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(a)
a = CGL.newDiscreteData(z, '{DiscreteData}')
d = CGL.newDataClass(a, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(a)
i = CGL.newBaseIterativeData(b, '{BaseIterativeData}')
d = CGL.newDataClass(i, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(i)
i = CGL.newZoneIterativeData(z, '{ZoneIterativeData}')
d = CGL.newDataClass(i, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(i)
m = CGL.newRigidGridMotion(z, '{RigidGridMotion}',
                           vector=NPY.array([[0.0, 0.0], [0.0, 0.0], [0.0, 0.0]]))
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(m)
m = CGL.newArbitraryGridMotion(z, '{ArbitraryGridMotion}')
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(m)
x = CGL.newZoneGridConnectivity(z)
x = CGL.newGridConnectivity(x, '{GridConnectivity}', z[0])
CGL.newPointRange(x, value=NPY.array([[1, 1], [1, 1], [1, 1]], dtype=NPY.int32, order='F'))
p = CGL.newGridConnectivityProperty(x)
m = CGL.newPeriodic(p)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(m)
w = CGL.newConvergenceHistory(b)
d = CGL.newDataClass(w, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(w)
i = CGL.newIntegralData(b, '{IntegralData}')
d = CGL.newDataClass(i, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(i)
i = CGL.newUserDefinedData(b, '{UserDefinedData}')
d = CGL.newDataClass(i, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(i)
i = CGL.newGravity(b)
d = CGL.newDataClass(i, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(i)
f = CGL.newFlowEquationSet(b)
d = CGL.newDataClass(f, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(f)
m = CGL.newGasModel(f)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
u = CGL.newDimensionalUnits(m)
m = CGL.newViscosityModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newThermalConductivityModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newThermalRelaxationModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newChemicalKineticsModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newTurbulenceClosure(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newTurbulenceModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newEMConductivityModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newEMMagneticFieldModel(f)
u = CGL.newDimensionalUnits(m)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
m = CGL.newEMElectricFieldModel(f)
d = CGL.newDataClass(m, CGK.NormalizedByDimensional_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'dataclass dimensional without dimensionalunits'
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newFamily(b, '{Family}')
z = CGL.newZone(b, '{Zone}', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
g = CGL.newGridCoordinates(z, CGK.GridCoordinates_s)
w = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
f = CGL.newFlowSolution(z)
a = CGL.newDataArray(f, '{DataArray}', value=NPY.ones((4, 6, 8), dtype='float64', order='F'))
d = CGL.newDataClass(a, CGK.NormalizedByDimensional_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'exponents without dimensionalunits'
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newFamily(b, '{Family}')
z = CGL.newZone(b, '{Zone}', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
g = CGL.newGridCoordinates(z, CGK.GridCoordinates_s)
w = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
f = CGL.newFlowSolution(z)
a = CGL.newDataArray(f, '{DataArray}', value=NPY.ones((4, 6, 8), dtype='float64', order='F'))
d = CGL.newDimensionalExponents(a)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'dimensionalunits without dataclass'
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newFamily(b, '{Family}')
z = CGL.newZone(b, '{Zone}', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
g = CGL.newGridCoordinates(z, CGK.GridCoordinates_s)
w = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
f = CGL.newFlowSolution(z)
a = CGL.newDataArray(f, '{DataArray}', value=NPY.ones((4, 6, 8), dtype='float64', order='F'))
d = CGL.newDimensionalUnits(a)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'bad dimensionalunits #1'
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base}', 3, 3)
d = CGL.newFamily(b, '{Family}')
z = CGL.newZone(b, '{Zone}', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
g = CGL.newGridCoordinates(z, CGK.GridCoordinates_s)
w = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
w = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype='float64', order='F'))
f = CGL.newFlowSolution(z)
a = CGL.newDataArray(f, '{DataArray}', value=NPY.ones((4, 6, 8), dtype='float64', order='F'))
d = CGL.newDimensionalUnits(a)
d[1] = 'Meter'
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
# This file has been generated on [2021-05-12 08:46:29]
# Using platform [Windows fv-az122-648 Intel64 Family 6 Model 63 Stepping 2, GenuineIntel]

DATE='2021-05-12 08:46:29'
PLATFORM='Windows fv-az122-648 Intel64 Family 6 Model 63 Stepping 2, GenuineIntel'
PFX='#'

HAS_HDF5=True
HAS_H5PY=0
HAS_PY2=0
HAS_PY3=1
HAS_VTK=True
HAS_MSW=1
USE_COMPACT_STORAGE=1

INCLUDE_DIRS=['D:\\bld\\pycgns_1620808498069\\_h_env\\lib\\site-packages\\numpy\\core\\include', 'D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\include', 'D:\\bld\\pycgns_1620808498069\\_h_env/lib/python3.9/site-packages/numpy/core/include', '', 'D:\\bld\\pycgns_1620808498069\\_h_env/include', '../include', '/include']
LIBRARY_DIRS=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\lib', '', 'D:\\bld\\pycgns_1620808498069\\_h_env/lib', '../lib', '/lib64', '/lib']

HDF5_VERSION='1.10.6'
HDF5_PATH_INCLUDES=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\include']
HDF5_PATH_LIBRARIES=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\lib']
HDF5_LINK_LIBRARIES=['hdf5']
HDF5_EXTRA_ARGS=[]

NUMPY_VERSION='1.19.5'
NUMPY_VERSION_API='0x01000009'
NUMPY_PATH_INCLUDES=['D:\\bld\\pycgns_1620808498069\\_h_env\\lib\\site-packages\\numpy\\core\\include']
NUMPY_PATH_LIBRARIES=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\lib']
NUMPY_LINK_LIBRARIES=[]
NUMPY_EXTRA_ARGS=[]

PYQT_VERSION='1.9.0'
QT_VERSION='5.12.9'
CYTHON_VERSION='0.29.23'
VTK_VERSION='9.0.1'

__version__=5 # @@UPDATEVERSION@@
__release__=3 # @@UPDATERELEASE@@
__revision__=1 # @@UPDATEREVISION@@
__vid__="%s.%s.%s"%(__version__,__release__,__revision__)
__doc__='pyCGNS - %s - Python package for CFD General Notation System'        %(__vid__)
version=__vid__

NAME='pyCGNS'
VERSION=__vid__
DESCRIPTION=__doc__
AUTHOR='Marc Poinot'
EMAIL="marc.poinot@safrangroup.com"
LICENSE="LGPL 2"




#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
"""
 We are changing for a full h5py implementation, default switch is in
 config: HAS_H5PY forces h5py instead of raw HDF5 C API implementation

 CGNS.MAP module is the interface to CGNS/HDF5 (formerly was CHLone)
  - now all the CHLone code is embedded in CGNS.MAP (EmbeddedCHLone)
  - you DO NOT need to install CHLone on its own

  Flags: 

  - python does NOT have bitfiled or unseigned int that would act as
    bit masks we have in C. So each operation on such bitfields (flags)
    should be re-masked against the actual 32bits size of the flags,
    which means you should always have a & flags.ALL in your operation
"""
#
from CGNS.pyCGNSconfig import HAS_H5PY

if HAS_H5PY:
  from .cgio import load, save, probe, flags

  NONE = flags.NONE
  ALL = flags.ALL
  TRACE = flags.TRACE
  FOLLOWLINKS = flags.FOLLOWLINKS
  DEBUG = flags.DEBUG
  NODATA = flags.NODATA
  MERGELINKS = 0
  COMPRESS = flags.COMPRESS
  REVERSEDIMS = flags.REVERSEDIMS
  OWNDATA = flags.OWNDATA
  UPDATE = flags.UPDATE
  DELETEMISSING = flags.DELETEMISSING
  ALTERNATESIDS = flags.ALTERNATESIDS
  NOTRANSPOSE = 0
  UPDATEONLY = flags.UPDATEONLY
  FORTRANFLAG = flags.FORTRANFLAG
  PROPAGATE = flags.PROPAGATE
  DEFAULT = flags.DEFAULT
  DEFAULTS = flags.DEFAULTS
  KEEPLIST = flags.KEEPLIST
  CHECKSUM = flags.CHECKSUM

  LKOK = flags.links.OK
  LKFAIL = flags.links.FAIL
  LKBADSYNTAX = flags.links.BADSYNTAX
  LKNOFILE = flags.links.NOFILE
  LKFILENOREAD = flags.links.FILENOREAD
  LKNONODE = flags.links.NONODE
  LKLOOP = flags.links.LOOP
  LKIGNORED = flags.links.IGNORED
  
else:
  from .EmbeddedCHLone import load, save, probe
  from .EmbeddedCHLone import CHLoneException as error
  #
  from . import EmbeddedCHLone as CHL
  #
  S2P_NONE = CHL.FNONE
  S2P_ALL = CHL.FALL
  S2P_TRACE = CHL.FTRACE
  S2P_FOLLOWLINKS = CHL.FFOLLOWLINKS
  S2P_DEBUG = CHL.FDEBUG
  S2P_NODATA = CHL.FNODATA
  S2P_MERGELINKS = 0
  S2P_COMPRESS = CHL.FCOMPRESS
  S2P_REVERSEDIMS = CHL.FREVERSEDIMS
  S2P_OWNDATA = CHL.FOWNDATA
  S2P_UPDATE = CHL.FUPDATE
  S2P_DELETEMISSING = CHL.FDELETEMISSING
  S2P_ALTERNATESIDS = CHL.FALTERNATESIDS
  S2P_NOTRANSPOSE = 0
  S2P_UPDATEONLY = CHL.FUPDATEONLY
  S2P_FORTRANFLAG = CHL.FFORTRANFLAG
  S2P_PROPAGATE = CHL.FPROPAGATE
  S2P_DEFAULT = CHL.FDEFAULT
  S2P_DEFAULTS = CHL.FDEFAULTS
  S2P_KEEPLIST = CHL.FKEEPLIST
  S2P_CHECKSUM = CHL.FCHECKSUM

  S2P_LKOK = CHL.LKOK
  S2P_LKFAIL = CHL.LKFAIL
  S2P_LKBADSYNTAX = CHL.LKBADSYNTAX
  S2P_LKNOFILE = CHL.LKNOFILE
  S2P_LKFILENOREAD = CHL.LKFILENOREAD
  S2P_LKNONODE = CHL.LKNONODE
  S2P_LKLOOP = CHL.LKLOOP
  S2P_LKIGNORED = CHL.LKIGNORED

  NONE = CHL.FNONE
  ALL = CHL.FALL
  TRACE = CHL.FTRACE
  FOLLOWLINKS = CHL.FFOLLOWLINKS
  DEBUG = CHL.FDEBUG
  NODATA = CHL.FNODATA
  MERGELINKS = 0
  COMPRESS = CHL.FCOMPRESS
  REVERSEDIMS = CHL.FREVERSEDIMS
  OWNDATA = CHL.FOWNDATA
  UPDATE = CHL.FUPDATE
  DELETEMISSING = CHL.FDELETEMISSING
  ALTERNATESIDS = CHL.FALTERNATESIDS
  NOTRANSPOSE = 0
  UPDATEONLY = CHL.FUPDATEONLY
  FORTRANFLAG = CHL.FFORTRANFLAG
  PROPAGATE = CHL.FPROPAGATE
  DEFAULT = CHL.FDEFAULT
  DEFAULTS = CHL.FDEFAULTS
  KEEPLIST = CHL.FKEEPLIST
  CHECKSUM = CHL.FCHECKSUM

  LKOK = CHL.LKOK
  LKFAIL = CHL.LKFAIL
  LKBADSYNTAX = CHL.LKBADSYNTAX
  LKNOFILE = CHL.LKNOFILE
  LKFILENOREAD = CHL.LKFILENOREAD
  LKNONODE = CHL.LKNONODE
  LKLOOP = CHL.LKLOOP
  LKIGNORED = CHL.LKIGNORED

# --- forward compat stuff here
  class Flags(object):
    pass

  flags = Flags()
  flags.links = Flags()

  flags.NONE = NONE
  flags.ALL = ALL
  flags.TRACE = TRACE
  flags.FOLLOWLINKS = FOLLOWLINKS
  flags.DEBUG = DEBUG
  flags.NODATA = NODATA
  flags.MERGELINKS = 0
  flags.COMPRESS = COMPRESS
  flags.REVERSEDIMS = REVERSEDIMS
  flags.OWNDATA = OWNDATA
  flags.UPDATE = UPDATE
  flags.DELETEMISSING = DELETEMISSING
  flags.ALTERNATESIDS = ALTERNATESIDS
  flags.NOTRANSPOSE = 0
  flags.UPDATEONLY = UPDATEONLY
  flags.FORTRANFLAG = FORTRANFLAG
  flags.PROPAGATE = PROPAGATE
  flags.DEFAULT = DEFAULT
  flags.DEFAULTS = DEFAULTS
  flags.KEEPLIST = KEEPLIST
  flags.CHECKSUM = CHECKSUM

  flags.links.OK = LKOK
  flags.links.FAIL = LKFAIL
  flags.links.BADSYNTAX = LKBADSYNTAX
  flags.links.NOFILE = LKNOFILE
  flags.links.FILENOREAD = LKFILENOREAD
  flags.links.NONODE = LKNONODE
  flags.links.LOOP = LKLOOP
  flags.links.IGNORED = LKIGNORED


def flags_set(source=flags.DEFAULT, flag=flags.NONE):
  return source | flag & flags.ALL

def flags_unset(source, flag):
  return source & ~flag & flags.ALL

def flags_check(source, flag):
  return source & flag == flag

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.MAP         as CGM
from   CGNS.APP.lib import mergeTrees

import numpy as NPY


def test():
    T1 = CGL.newCGNSTree()
    b1 = CGL.newCGNSBase(T1, 'B1', 3, 3)
    b2 = CGL.newCGNSBase(T1, 'B2', 3, 3)
    f1 = CGL.newFamily(b1, 'F1')
    f2 = CGL.newFamily(b1, 'F2')
    f1 = CGL.newFamily(b2, 'F1')
    f2 = CGL.newFamily(b2, 'F2')
    f3 = CGL.newFamily(b2, 'F3')
    f4 = CGL.newFamily(b2, 'F4')
    z1 = CGL.newZone(b1, 'Z1', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z2 = CGL.newZone(b1, 'Z2', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z3 = CGL.newZone(b1, 'Z3', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z4 = CGL.newZone(b1, 'Z4', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z5 = CGL.newZone(b1, 'Z5', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z1 = CGL.newZone(b2, 'Z1', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z2 = CGL.newZone(b2, 'Z2', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z3 = CGL.newZone(b2, 'Z3', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))

    T2 = CGL.newCGNSTree()
    b1 = CGL.newCGNSBase(T2, 'B1', 3, 3)
    b2 = CGL.newCGNSBase(T2, 'B2', 3, 3)
    b3 = CGL.newCGNSBase(T2, 'B3', 2, 2)
    f1 = CGL.newFamily(b1, 'F1')
    f1 = CGL.newFamily(b2, 'F1')
    f2 = CGL.newFamily(b2, 'F2')
    f3 = CGL.newFamily(b2, 'F3')
    f4 = CGL.newFamily(b2, 'F4')
    f5 = CGL.newFamily(b2, 'F5')
    f1 = CGL.newFamily(b3, 'F1')
    f2 = CGL.newFamily(b3, 'F2')
    z1 = CGL.newZone(b1, 'Z1', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z2 = CGL.newZone(b1, 'Z2', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z3 = CGL.newZone(b1, 'Z3', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z1 = CGL.newZone(b2, 'Z1', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z2 = CGL.newZone(b2, 'Z2', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z3 = CGL.newZone(b2, 'Z3', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))
    z4 = CGL.newZone(b2, 'Z4', NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order='F'))

    T3 = CGL.newCGNSTree()
    b1 = CGL.newCGNSBase(T3, 'B1', 3, 3)
    b2 = CGL.newCGNSBase(T3, 'B7', 3, 3)
    b3 = CGL.newCGNSBase(T3, 'B8', 2, 2)
    f1 = CGL.newFamily(b1, 'F1')
    f1 = CGL.newFamily(b2, 'F8')

    Tlist = []
    for TT in [T1, T2, T3]: Tlist.append(TT)

    count = 0
    for t in Tlist:
        CGM.save('merge-T%.2d.hdf' % count, t, flags=CGM.S2P_TRACE)
        count += 1
    T = mergeTrees(Tlist)
    CGM.save('merge-Result.hdf', T, flags=CGM.S2P_TRACE)
    return T

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'bc'
diag = True


def makeCorrectTree(vertexsize, cellsize, ntris):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, 'Base', 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype='int32', order='F')
    z = CGL.newZone(b, 'Zone', s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype='float64', order='F'))
    tetras = CGL.newElements(z, 'TETRAS', CGK.TETRA_4_s,
                             NPY.array([[1, cellsize]], 'i', order='F'),
                             NPY.ones((cellsize * 4), dtype='int32'))
    tris = CGL.newElements(z, 'TRIS', CGK.TRI_3_s,
                           NPY.array([[cellsize + 1, cellsize + ntris]], 'i', order='F'),
                           NPY.ones((ntris * 3), dtype='int32'))
    zbc = CGL.newZoneBC(z)
    n = CGL.newBoundary(zbc, 'BC', [range(cellsize + 1, cellsize + ntris + 1)], btype=CGK.Null_s, family=None,
                        pttype=CGK.PointList_s)
    g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
    return (T, b, z, zbc, n, g)


vertexsize = 20
cellsize = 7
ntris = 12
(T, b, z, zbc, n, g) = makeCorrectTree(vertexsize, cellsize, ntris)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'bc bad location'
diag = False
(T, b, z, zbc, n, g) = makeCorrectTree(vertexsize, cellsize, ntris)
CGL.newPointRange(n, value=NPY.array([[1, cellsize]], 'i'))
zbc[2] = []
n = CGL.newBoundary(zbc, 'BC', [range(1, cellsize + 1)], btype=CGK.Null_s, family=None, pttype=CGK.PointList_s)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'bc both PointList and PointRange'
diag = False
(T, b, z, zbc, n, g) = makeCorrectTree(vertexsize, cellsize, ntris)
CGL.newPointRange(n, value=NPY.array([[cellsize + 1, cellsize + ntris]], 'i'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'bc no PointList or PointRange'
diag = False
(T, b, z, zbc, n, g) = makeCorrectTree(vertexsize, cellsize, ntris)
CGU.removeChildByName(n, CGK.PointList_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'bc FamilySpecified but not FamilyName'
diag = False
(T, b, z, zbc, n, g) = makeCorrectTree(vertexsize, cellsize, ntris)
n[1] = CGU.setStringAsArray(CGK.FamilySpecified_s)
TESTS.append((tag, T, diag))



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from __future__ import division
from builtins import (str, bytes, range, dict)

from CGNS.NAV.moption import Q7OptionContext as OCTXT

import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK

from qtpy.QtCore import Qt, QModelIndex
from qtpy.QtWidgets import QStyledItemDelegate

from CGNS.NAV.Q7FormWindow import Ui_Q7FormWindow
from CGNS.NAV.mtable import Q7TableModel
from CGNS.NAV.wstylesheets import Q7TABLEVIEWSTYLESHEET
from CGNS.NAV.wfingerprint import Q7Window


# -----------------------------------------------------------------
def divpairs(n):
    d = n
    l = []
    while d != 0:
        m = n // d
        r = n % d
        if r == 0:
            l += [(d, m)]
        d -= 1
    return l


# -----------------------------------------------------------------
class Q7TableItemDelegate(QStyledItemDelegate):
    def paint(self, painter, option, index):
        t = index.data(Qt.DisplayRole)
        r = option.rect.adjusted(2, 2, -2, -2)
        painter.drawText(r, Qt.AlignVCenter | Qt.AlignLeft, t, r)
        # QStyledItemDelegate.paint(self, painter, option, index)


# -----------------------------------------------------------------
class Q7Form(Q7Window, Ui_Q7FormWindow):
    def __init__(self, control, node, fgprintindex):
        Q7Window.__init__(self, Q7Window.VIEW_FORM, control,
                          node.sidsPath(), fgprintindex)
        self._node = node
        self._operatorlist = ['']
        for t in self._node.sidsTypeList():
            self.eType.addItem(t)
        self.bClose.clicked.connect(self.reject)
        self.bInfo.clicked.connect(self.infoFormView)
        self.cFortranOrderOff.setChecked(False)
        self.cFortranOrderOff.setDisabled(True)
        for dt in node.sidsDataType(all=True):
            self.cDataType.addItem(dt)

    def infoFormView(self):
        self._control.helpWindow('Form')

    def updateTreeStatus(self):
        pass

    def accept(self):
        pass

    def reject(self):
        self.close()

    def show(self):
        self.reset()
        super(Q7Form, self).show()

    def reset(self):
        self.eName.setText(self._node.sidsName())
        self.ePath.setText(self._node.sidsPath())
        self.ePath.setReadOnly(True)
        self.sMinH.setRange(-999999999, 999999999)
        self.sMinV.setRange(-999999999, 999999999)
        self.sMinH.setValue(1)
        self.sMinV.setValue(1)
        self.setCurrentType(self._node.sidsType())
        dims = self._node.sidsDims()
        its = 1
        for x in dims:
            its *= x
        ndz = its * self._node.sidsDataTypeSize()
        self.eDims.setText(str(dims))
        self.setDataType(self._node)
        self.eItems.setText(str(its))
        self.ls = 1
        self.cs = dims[0]
        if len(dims) > 1:
            self.ls = 1
            for x in dims[1:]:
                self.ls *= x
        self.showparams = {'cs': self.cs, 'ls': self.ls, 'mode': 'IJ'}
        for dp in divpairs(self.cs * self.ls):
            self.cRowColSize.addItem("%d x %d" % dp)
        ix = self.cRowColSize.findText("%d x %d" % (self.cs, self.ls))
        self.cRowColSize.setCurrentIndex(ix)
        self.model = Q7TableModel(self._node, self.showparams)
        self.tableView.setModel(self.model)
        self.tableView.setStyleSheet(self._stylesheet)
        self.bMinimize.clicked.connect(self.minimizeCells)
        #
        self.tableView.clicked[QModelIndex].connect(self.clickedNode)
        self.cRowColSize.currentIndexChanged[int].connect(self.resizeTable)
        self.sMinH.valueChanged[int].connect(self.resetIndex)
        self.sMinV.valueChanged[int].connect(self.resetIndex)
        # QObject.connect(self.tableView,
        #                SIGNAL("clicked(QModelIndex)"),
        #                self.clickedNode)
        # QObject.connect(self.cRowColSize,
        #                SIGNAL("currentIndexChanged(int)"),
        #                self.resizeTable)
        # QObject.connect(self.sMinH,
        #                SIGNAL("valueChanged(int)"),
        #                self.resetIndex)
        # QObject.connect(self.sMinV,
        #                SIGNAL("valueChanged(int)"),
        #                self.resetIndex)
        lk = self.FG.isLink(self._node.sidsPath())
        if lk:
            self.setLabel(self.eDestDir, lk[0])
            self.setLabel(self.eDestFile, lk[1])
            self.setLabel(self.eDestPath, lk[2])
        else:
            self.tFiles.setDisabled(True)
        self.tPython.setDisabled(True)
        if self._node.sidsDataType() == CGK.C1:
            if self._node.sidsValue().ndim > 1:
                txt = self._node.sidsValue().T.tostring().decode('ascii')
                self.cFortranOrderOff.setChecked(True)
                self.cFortranOrderOff.setDisabled(True)
            else:
                txt = self._node.sidsValue().tostring().decode('ascii')
            self.eText.initText(txt)

    def resizeTable(self):
        s = self.cRowColSize.currentText()
        (r, c) = s.split('x')
        self.cs = int(r)
        self.ls = int(c)
        self.showparams = {'cs': self.cs, 'ls': self.ls, 'mode': 'IJ'}
        self.model = Q7TableModel(self._node, self.showparams)
        self.tableView.setModel(self.model)
        self.tableView.setStyleSheet(self._stylesheet)

    def clickedNode(self, index):
        pass

    def minimizeCells(self, *args):
        self.tableView.resizeColumnsToContents()
        self.tableView.resizeRowsToContents()

    def setCurrentType(self, ntype):
        idx = self.eType.findText(ntype)
        self.eType.setCurrentIndex(idx)

    def setDataType(self, node):
        dt = node.sidsDataType()
        ix = self.cDataType.findText(dt)
        if ix == -1:
            return
        self.cDataType.setCurrentIndex(ix)

    def resetIndex(self):
        hm = self.sMinH.value()
        vm = self.sMinV.value()
        self.model.setRange(hm, vm)
        self.model.headerDataChanged.emit(Qt.Vertical, 1, 1)
        self.model.headerDataChanged.emit(Qt.Horizontal, 1, 1)

    def addControlLine(self):
        pass

    def doRelease(self):
        pass

# -----------------------------------------------------------------



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  ---------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib      as C
import CGNS.PAT.cgnserrors   as E
import CGNS.PAT.cgnskeywords as K
import numpy             as N

data = C.newAdditionalFamilyName(None, family='{AdditionalFamilyName}')
status = '-'
comment = 'SIDS Leaf node'
pattern = [data, status, comment]



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = 'flow equation set'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base#001}', 3, 3)
f = CGL.newFlowEquationSet(b)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'governing equations'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base#001}', 3, 3)
f = CGL.newFlowEquationSet(b)
g = CGL.newGoverningEquations(f)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'diffusion model'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base#001}', 3, 3)
f = CGL.newFlowEquationSet(b)
g = CGL.newGoverningEquations(f)
d = CGL.newDiffusionModel(g, NPY.ones(6, dtype='int32'))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = 'flow equation set all models #1'
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, '{Base#001}', 3, 3)
f = CGL.newFlowEquationSet(b)
g = CGL.newGoverningEquations(f)
d = CGL.newDiffusionModel(g, NPY.zeros(6, dtype='int32'))
m = CGL.newGasModel(f)
m = CGL.newThermalConductivityModel(f)
m = CGL.newViscosityModel(f)
m = CGL.newTurbulenceModel(f)
m = CGL.newTurbulenceClosure(f)
m = CGL.newThermalRelaxationModel(f)
m = CGL.newChemicalKineticsModel(f)
m = CGL.newEMElectricFieldModel(f)
m = CGL.newEMMagneticFieldModel(f)
m = CGL.newEMConductivityModel(f)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
# TESTING RAW HDF5/C API and H5PY IMPLEMENTATIONS ***
# - test save first
# - test load

from __future__ import unicode_literals
from __future__ import print_function
from builtins import (bytes, str, range, dict)
import os
import subprocess
import unittest

import numpy

def genTrees():
    import CGNS.PAT.cgnskeywords as CGK
    import CGNS.PAT.cgnslib as CGL

    tree = CGL.newCGNSTree()
    b = CGL.newBase(tree, '{Base}', 2, 3)
    z = CGL.newZone(b, '{Zone}', numpy.array([[5, 4, 0], [7, 6, 0]], order='F'))
    g = CGL.newGridCoordinates(z, 'GridCoordinates')
    d = CGL.newDataArray(g, CGK.CoordinateX_s, numpy.ones((5, 7), dtype='d', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateY_s, numpy.ones((5, 7), dtype='d', order='F'))
    d = CGL.newDataArray(g, CGK.CoordinateZ_s, numpy.ones((5, 7), dtype='d', order='F'))
    return (tree,)

class MAPTestCase(unittest.TestCase):
    def setUp(self):
        try:
            self.T = genTrees()[0]
            self.L = []
        except ImportError:
            self.T = []
            self.L = []
        self.HDF01 = 'T01.hdf'
        self.HDF02 = 'T02.hdf'

    def unlink(self, filename):
        if os.path.isfile(filename):
            os.unlink(filename)

    def chmod(self, filename, mode):
        if os.path.isfile(filename):
            os.chmod(filename, mode)

    def getDump(self, filename, path, format, fct):
        com = "h5dump -d \"%s/ data%s\" %s" % (path, format, filename)
        r = subprocess.check_output(com, shell=True)
        r = r.decode('ascii')
        d = r.split('\n')[10]
        v = d.split('): ')[1:][0]
        return fct(v)

    def void_test_000_Constants(self):
        import CGNS
        import CGNS.MAP
        from CGNS.MAP import flags, flags_set, flags_unset, flags_check    
        f1 = flags.NONE
        f2 = flags.ALL
        self.assertEqual(flags.NONE & flags.ALL, flags.NONE)
        self.assertEqual(flags.NONE | flags.ALL & flags.ALL, flags.ALL)
        self.assertEqual(flags.ALL | ~flags.NONE & flags.ALL, flags.ALL)
        self.assertEqual(flags.ALL ^ flags.TRACE & flags.ALL, flags.ALL & ~flags.TRACE)
        l = flags.links.OK
        f = flags_set()
        self.assertEqual(f, flags.DEFAULT)
        f = flags_set(f, flags.TRACE)
        self.assertEqual(f, flags.DEFAULT | flags.TRACE & flags.ALL)
        c = flags_check(f, flags.TRACE)
        self.assertTrue(c)
        f = flags_unset(f, flags.TRACE)
        self.assertEqual(f, flags.DEFAULT & flags.ALL)
        c = flags_check(f, flags.TRACE)
        self.assertFalse(c)

    def void_test_001_Probe(self):
        from CGNS.MAP import probe
        from CGNS.PAT.cgnserrors import CGNSException
        self.assertRaisesRegex(CGNSException,
                                "[900].*", probe, "/Z/ ?/ /U.cgns")
    
    def void_test_002_Save_Args(self):
        from CGNS.MAP import save, flags
        from CGNS.PAT.cgnserrors import CGNSException
        self.assertRaisesRegex(CGNSException,
                                "[910].*", save, self.HDF01, self.T, zflag=None)
        self.assertRaisesRegex(CGNSException,
                                "[917].*", save, self.HDF01, self.T,
                                links=[['A', 'A', 'A']])        
        self.unlink(self.HDF01)
        self.assertRaisesRegex(CGNSException, "[906].*",
                                save, self.HDF01, None)
        self.unlink(self.HDF01)
        save(self.HDF01, self.T)
        v = self.getDump(self.HDF01, '/{Base}/{Zone}', '[0,1]', int)
        self.assertEqual(v, 7)
        v = self.getDump(self.HDF01, '/{Base}/{Zone}', '[1,1]', int)
        self.assertEqual(v, 6)
        self.unlink(self.HDF02)
        save(self.HDF02, self.T)
        self.chmod(self.HDF02, 0)
        self.assertRaisesRegex(CGNSException,
                                "[100].*", save, self.HDF02,
                                self.T, flags=flags.UPDATE)
        self.chmod(self.HDF02, 511)
        self.unlink(self.HDF02)
        self.unlink(self.HDF01)
        self.assertRaisesRegex(CGNSException,
                                "[300].*", save, self.HDF01,
                                [None, None, self.T, None])
        self.unlink(self.HDF01)
        flags = flags.DEFAULT | flags.UPDATE
        self.assertRaisesRegex(CGNSException,
                                "[100].*", save, self.HDF01, self.T, flags=flags)

    def void_test_003_Load_Args(self):
        from CGNS.MAP import load
        from CGNS.PAT.cgnserrors import CGNSException
        self.assertRaisesRegex(CGNSException,
                                "[910].*", load, self.HDF01, zflag=None)
        self.assertRaisesRegex(CGNSException,
                                "[911].*", load, self.HDF01, flags=[])
        self.assertRaisesRegex(CGNSException,
                                "[912].*", load, self.HDF01, depth=[])
        self.assertRaisesRegex(CGNSException,
                                "[909].*", load, self.HDF01,
                                maxdata=400, threshold=200)
        self.assertRaisesRegex(CGNSException,
                                "[908].*", load, self.HDF01,
                                flags=flags.DEFAULT, maxdata=400)
        self.assertRaisesRegex(CGNSException,
                                "[100].*", load, 'foo.hdf')
        self.assertRaisesRegex(CGNSException,
                                "[101].*", load, 'chltest.py')

    def void_test_004_LoadSave(self):
        from CGNS.MAP import load
        from CGNS.PAT.cgnserrors import CGNSException
        save(self.HDF01, self.T)
        (t, l, p) = load(self.HDF01)

# ---
print('-' * 70 + '\nCGNS.MAP test suite')
suite = unittest.TestLoader().loadTestsFromTestCase(MAPTestCase)
unittest.TextTestRunner(verbosity=2).run(suite)

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from __future__ import unicode_literals
from builtins import (str, bytes, range, dict)

from CGNS.NAV.moption import Q7OptionContext as OCTXT

import os.path
import stat
import string
import time
import sys

from CGNS.pyCGNSconfig import HAS_PY2, HAS_MSW

from qtpy.QtCore import (Qt, QSortFilterProxyModel, QModelIndex, QFileInfo)
from qtpy.QtWidgets import (QWidget, QFileIconProvider, QAbstractItemView, QFileSystemModel)
from qtpy.QtGui import QIcon, QPixmap

from CGNS.NAV.Q7FileWindow import Ui_Q7FileWindow

import CGNS.NAV.wmessages as MSG

LOADBUTTON = ['Load', 'Load from selected file']
SAVEBUTTON = ['Save', 'Save to selected file']
(LOADMODE, SAVEMODE) = (0, 1)


# -----------------------------------------------------------------
def checkFilePermission(path, write=False):
    if HAS_MSW:
        return True
    if not (os.path.isfile(path) or os.path.isdir(path)):
        return False
    r = False
    w = False
    st = os.stat(path)
    m = st.st_mode
    if (st.st_uid == os.getuid()) and (m & stat.S_IRUSR):
        r = True
    if (st.st_gid == os.getgid()) and (m & stat.S_IRGRP):
        r = True
    if m & stat.S_IROTH:
        r = True
    if not r:
        return False
    if (st.st_uid == os.getuid()) and (m & stat.S_IWUSR):
        w = True
    if (st.st_gid == os.getgid()) and (m & stat.S_IWGRP):
        w = True
    if m & stat.S_IWOTH:
        w = True
    if write and not w:
        return False
    return True


# -----------------------------------------------------------------
class Q7FileFilterProxy(QSortFilterProxyModel):
    def __init__(self, parent):
        super(Q7FileFilterProxy, self).__init__(parent)
        # QSortFilterProxyModel.__init__(self, parent)
        self.model = parent.model
        self.treeview = parent.treeview
        self.wparent = parent
        self.setDynamicSortFilter(True)
        import locale
        locale.setlocale(locale.LC_ALL, 'C')

    def filterAcceptsRow(self, row, parentindex):
        idx = self.model.index(row, 1, parentindex)
        p = self.model.filePath(idx)
        if isinstance(p, bytes):
            p = str(self.model.filePath(idx).decode('utf-8'))
        elif HAS_PY2:
            if isinstance(p, unicode):
                p = str(p)
        if not self.checkPermission(p):
            return False
        if os.path.isdir(p):
            if self.wparent.cShowDirs.checkState() != Qt.Checked:
                if len(p) > len(self.wparent.selecteddir):
                    return False
            return True
        self.wparent.getBoxes()
        # if (self.wparent.cShowAll.checkState()==Qt.Checked): xlist=[]
        r = self.wparent.parent.matchFileExtensions(p)
        return r

    def checkPermission(self, path, write=False):
        return checkFilePermission(path, write)

    def lessThan(self, left, right):
        c = self.sortColumn()
        a = self.model.data(left)
        b = self.model.data(right)
        if c in (0, 2):
            return a < b
        if c == 3:
            fmt_qt = "dd-MM-yyyy hh:mm:ss"
            left_time = self.model.lastModified(left)
            right_time = self.model.lastModified(right)
            a = left_time.toString(fmt_qt)
            b = right_time.toString(fmt_qt)
            fmtr = "%d-%m-%Y %H:%M:%S"
            fmtw = "%Y-%m-%d %H:%M:%S"
            ad = time.strptime(str(a), fmtr)
            bd = time.strptime(str(b), fmtr)
            af = time.strftime(fmtw, ad)
            bf = time.strftime(fmtw, bd)
            return af < bf
        if c == 1:
            wg = {'MB': 1e3, 'GB': 1e6, 'KB': 1}
            try:
                (av, au) = a.split()
                (bv, bu) = b.split()
            except ValueError:
                return a < b
            av = float(av.replace(',', '.')) * wg[au]
            bv = float(bv.replace(',', '.')) * wg[bu]
            return av < bv
        return 1


# -----------------------------------------------------------------
class Q7FileIconProvider(QFileIconProvider):
    slist = ['hdf', 'HDF', 'cgns', 'CGNS', 'adf', 'ADF']

    def __init__(self):
        super(Q7FileIconProvider, self).__init__()
        self.dir = QIcon(QPixmap(":/images/icons/folder.png"))
        self.cgns = QIcon(QPixmap(":/images/icons/tree-load.png"))
        self.empty = QIcon()

    def icon(self, fileinfo):
        if not isinstance(fileinfo, QFileInfo):
            return self.empty
        if fileinfo.isDir():
            return self.dir
        if fileinfo.suffix() in Q7FileIconProvider.slist:
            return self.cgns
        return self.empty


# -----------------------------------------------------------------
class Q7File(QWidget, Ui_Q7FileWindow):
    def __init__(self, parent, mode=LOADMODE):
        super(Q7File, self).__init__()
        self.setupUi(self)
        self.setWindowTitle("Load/Save")
        self.parent = parent
        self.iconProvider = Q7FileIconProvider()
        self.model = QFileSystemModel()
        self.model.setIconProvider(self.iconProvider)
        self.proxy = Q7FileFilterProxy(self)
        self.proxy.setSourceModel(self.model)
        self.treeview.setModel(self.proxy)
        self.setAttribute(Qt.WA_DeleteOnClose)
        #
        if HAS_PY2:
            self.model.directoryLoaded[unicode].connect(self.expandCols)
        else:
            self.model.directoryLoaded[str].connect(self.expandCols)
        self.treeview.expanded[QModelIndex].connect(self.expandCols)
        self.treeview.clicked[QModelIndex].connect(self.clickedNode)
        self.treeview.doubleClicked[QModelIndex].connect(self.clickedNodeAndLoad)
        self.direntries.lineEdit().returnPressed.connect(self.changeDirEdit)
        self.direntries.currentIndexChanged[int].connect(self.changeDirIndex)
        self.fileentries.currentIndexChanged[int].connect(self.changeFile)
        self.fileentries.lineEdit().editingFinished.connect(self.changeFile)
        self.tabs.currentChanged[int].connect(self.currentTabChanged)
        self.cShowAll.stateChanged[int].connect(self.updateView)
        self.cShowDirs.stateChanged[int].connect(self.updateView)
        self.rClearSelectedDirs.toggled[bool].connect(self.updateClearDirs)
        self.rClearSelectedFiles.toggled[bool].connect(self.updateClearFiles)
        self.rClearAllDirs.toggled[bool].connect(self.updateClearNone)
        self.rClearNoHDF.toggled[bool].connect(self.updateClearNoHDF)
        self.rClearNotFound.toggled[bool].connect(self.updateClearNotFound)
        #
        self.bClose.clicked.connect(self.closeUnlock)
        self.bCurrent.clicked.connect(self.currentDir)
        self.bBack.clicked.connect(self.backDir)
        self.bInfo.clicked.connect(self.infoFileView)
        self.bInfo2.clicked.connect(self.infoFileView)
        self.bClearHistory.clicked.connect(self.doClearHistory)
        self.mode = mode
        if self.mode == SAVEMODE:
            self.setMode(False)
        else:
            self.setMode(True)
        self.setBoxes()
        self.parent.getHistory()
        self.updateHistory()
        self.updateClearNotFound()

    def closeUnlock(self):
        self.parent.signals.cancel.emit()
        self.close()

    def closeEvent(self, event):
        self.closeUnlock()

    def infoFileView(self):
        self.parent.helpWindow('File')

    def updateView(self):
        p = self.direntries.currentText()
        self.setCurrentDir(p)
        print('CURRENT', p)
        # self.proxy.reset()
        self.proxy.beginResetModel()
        self.proxy.endResetModel()
        self.setCurrentDir(p)
        self.updateFileIfFound()

    def currentTabChanged(self, tabno):
        self.expandCols()
        self.setBoxes()

    def getOpt(self, name):
        return getattr(self, '_Ui_Q7FileWindow__O_' + name.lower())

    def setBoxes(self):
        ckh = self.getOpt('FilterHDFFiles')
        ckg = self.getOpt('FilterCGNSFiles')
        if self.parent.getOptionValue('FilterHDFFiles'):
            ckh.setCheckState(Qt.Checked)
        else:
            ckh.setCheckState(Qt.Unchecked)
        if self.parent.getOptionValue('FilterCGNSFiles'):
            ckg.setCheckState(Qt.Checked)
        else:
            ckg.setCheckState(Qt.Unchecked)
        if ((ckh.checkState() == Qt.Unchecked) and
                (ckg.checkState() == Qt.Unchecked)):
            self.cShowAll.setCheckState(Qt.Checked)

    def getBoxes(self):
        if self.getOpt('FilterHDFFiles').checkState() == Qt.Unchecked:
            self.parent.setOptionValue('FilterHDFFiles', False)
        else:
            self.parent.setOptionValue('FilterHDFFiles', True)
        if self.getOpt('FilterCGNSFiles').checkState() == Qt.Unchecked:
            self.parent.setOptionValue('FilterCGNSFiles', False)
        else:
            self.parent.setOptionValue('FilterCGNSFiles', True)

    def expandCols(self, *args):
        self.getBoxes()
        for n in range(3):
            self.treeview.resizeColumnToContents(n)

    def currentDir(self, *args):
        # p=os.path.split(self.path())[0]
        p = os.getcwd()
        self.setCurrentDir(p)

    def backDir(self, *args):
        p = os.path.split(self.path())[0]
        self.setCurrentDir(p)

    def changeDirEdit(self, *args):
        self.changeDir(args)

    def changeDirText(self, *args):
        self.changeDir(args)

    def changeDirIndex(self, *args):
        self.changeDir(args)

    def changeDir(self, *args):
        if self.updateMode:
            return
        p = str(self.direntries.currentText())
        if os.path.isdir(p):
            self.updateView()
        else:
            reply = MSG.wQuestion(self.parent, 110, 'Directory not found...',
                                  """The path below doesn't exist, do you want to remove<br>
                                     it from the history?<br>%s""" % p)
            if reply == MSG.OK:
                ix = self.direntries.currentIndex()
                self.direntries.removeItem(ix)

    def changeFile(self, *args):
        self.selectedfile = str(self.fileentries.lineEdit().text())
        d = None
        if self.cAutoDir.checkState() == Qt.Checked:
            d = self.parent.getHistoryFile(self.selectedfile)
        if d is not None:
            self.selecteddir = d[0]
            ix = self.direntries.findText(self.selecteddir)
            if ix != -1:
                self.direntries.setCurrentIndex(ix)
        else:
            self.selecteddir = self.direntries.lineEdit().text()
        self.updateFileIfFound()

    def selectedPath(self):
        return os.path.join(self.selecteddir, self.selectedfile)

    def updateFileIfFound(self):
        filepath = self.selectedPath()
        midx = self.model.index(filepath)
        if midx.row == 1:
            return
        fidx = self.proxy.mapFromSource(midx)
        if fidx.row == 1:
            return
        self.treeview.setCurrentIndex(fidx)
        self.treeview.scrollTo(fidx)

    def setCurrentDir(self, newpath):
        self.model.setRootPath(newpath)
        midx = self.model.index(newpath)
        fidx = self.proxy.mapFromSource(midx)
        self.treeview.setRootIndex(fidx)
        self.treeview.setCurrentIndex(fidx)
        self.direntries.setItemText(self.direntries.currentIndex(), newpath)
        self.selecteddir = newpath

    def setMode(self, load=True):
        if load:
            self.bAction.clicked.connect(self.load)
            self.bAction.setToolTip(LOADBUTTON[1])
            self.bAction.setText(LOADBUTTON[0])
            self.cOverwrite.setEnabled(False)
            self.cReadOnly.setEnabled(True)
            self.cNoLargeData.setEnabled(True)
            self.cFollowLinks.setEnabled(True)
            self.cDeleteMissing.setEnabled(False)
            if OCTXT.DoNotLoadLargeArrays:
                self.cNoLargeData.setCheckState(Qt.Checked)
            else:
                self.cNoLargeData.setCheckState(Qt.Unchecked)
            if OCTXT.FollowLinksAtLoad:
                self.cFollowLinks.setCheckState(Qt.Checked)
            else:
                self.cFollowLinks.setCheckState(Qt.Unchecked)
        else:
            self.bAction.clicked.connect(self.save)
            self.bAction.setToolTip(SAVEBUTTON[1])
            self.bAction.setText(SAVEBUTTON[0])
            self.cOverwrite.setEnabled(True)
            self.cReadOnly.setEnabled(False)
            self.cNoLargeData.setEnabled(False)
            self.cFollowLinks.setEnabled(False)
            self.cDeleteMissing.setEnabled(True)
            if OCTXT.FileUpdateRemovesChildren:
                self.cDeleteMissing.setCheckState(Qt.Checked)
            else:
                self.cDeleteMissing.setCheckState(Qt.Unchecked)

    def updateHistory(self):
        self.updateMode = True
        self.direntries.clear()
        self.fileentries.clear()
        hlist = self.parent.getHistory(fromfile=False)
        flist = []
        self.fileentries.addItem("")
        for i in list(hlist):
            if i != self.parent.getHistoryLastKey():
                self.direntries.addItem(str(i))
                flist = flist + hlist[i]
        for i in flist:
            self.fileentries.addItem(str(i))
        self.historyfiles = flist
        self.historydirs = list(hlist)
        if self.parent.getHistoryLastKey() in list(hlist):
            self.selecteddir = hlist[self.parent.getHistoryLastKey()][0]
            self.selectedfile = hlist[self.parent.getHistoryLastKey()][1]
            # ixd=self.direntries.findText(self.selecteddir)
            self.setCurrentDir(self.selecteddir)
            ixf = self.fileentries.findText(self.selectedfile)
            self.fileentries.setCurrentIndex(ixf)
            self.changeFile()
        else:
            self.selecteddir = os.getcwd()
            self.selectedfile = ""
            self.setCurrentDir(self.selecteddir)
        self.updateMode = False

    def doClearHistory(self):
        if self.rClearNoHDF.isChecked():
            reply = MSG.wQuestion(self.parent, 120, 'Clear history',
                                  """You really want to remove directory entries from history<br>
                                     where no file with defined extensions has been found?<br>""")
            if reply == MSG.OK:
                for d in self.parent.getDirNoHDFFromHistory():
                    self.parent.removeDirFromHistory(d)
                    self.updateHistory()
                    self.lClear.clear()
        if self.rClearNotFound.isChecked():
            reply = MSG.wQuestion(self.parent, 121, 'Clear history',
                                  """You really want to remove <b>NOT FOUND</b> entries from<br>
                                     the history of used directories?<br>""")
            if reply == MSG.OK:
                for d in self.parent.getDirNotFoundFromHistory():
                    self.parent.removeDirFromHistory(d)
                    self.updateHistory()
                    self.lClear.clear()
        if self.rClearAllDirs.isChecked():
            reply = MSG.wQuestion(self.parent, 122, 'Clear history',
                                  """You really want to remove <b>ALL</b> entries from the<br>
                                     the history of used files and directories?<br>""")
            if reply == MSG.OK:
                self.parent.destroyHistory()
                self.updateHistory()
        if self.rClearSelectedDirs.isChecked():
            for it in self.lClear.selectedItems():
                self.parent.removeDirFromHistory(it.text())
            self.updateHistory()
            self.updateClearDirs()
        if self.rClearSelectedFiles.isChecked():
            for it in self.lClear.selectedItems():
                fd = self.parent.getHistoryFile(it.text())
                if fd is not None:
                    self.parent.removeFileFromHistory(*fd)
            self.updateHistory()
            self.updateClearFiles()

    def updateClearNone(self):
        self.lClear.clear()

    def updateClearNoHDF(self):
        self.lClear.clear()
        self.lClear.setSelectionMode(QAbstractItemView.NoSelection)
        for d in self.parent.getDirNoHDFFromHistory():
            self.lClear.addItem(d)

    def updateClearNotFound(self):
        self.lClear.clear()
        self.lClear.setSelectionMode(QAbstractItemView.NoSelection)
        for d in self.parent.getDirNotFoundFromHistory():
            self.lClear.addItem(d)

    def updateClearDirs(self):
        self.lClear.clear()
        self.lClear.setSelectionMode(QAbstractItemView.MultiSelection)
        for d in self.historydirs:
            self.lClear.addItem(d)

    def updateClearFiles(self):
        self.lClear.clear()
        self.lClear.setSelectionMode(QAbstractItemView.MultiSelection)
        for d in self.historyfiles:
            self.lClear.addItem(d)

    def load(self):
        diag = self.checkTarget(self.selectedPath())
        if diag is None:
            self.parent.signals.buffer = self.selectedPath()
            self.hide()
            self.parent.signals.loadFile.emit()
            self.close()
        else:
            MSG.message("Load file: %s" % self.selectedPath(),
                        diag, MSG.INFO)

    def save(self):
        diag = self.checkTarget(self.selectedPath(), write=True)
        if diag is None:
            self.parent.signals.buffer = self.selectedPath()
            self.hide()
            self.parent.signals.saveFile.emit()
            self.close()
        else:
            MSG.message("Save file: %s" % self.selectedPath(),
                        diag, MSG.INFO)

    def checkTarget(self, filename, write=False):
        if os.path.isfile(filename) and not write:
            return None
        if not os.path.isfile(filename) and write:
            return None
        if self.cOverwrite.isChecked():
            sc = "The file is <b>completely replaced</b> by the current tree"
        else:
            sc = "The file is <b>updated</b> with the current tree contents."
            if self.cDeleteMissing.isChecked():
                sc += "The nodes <b>NOT</b> found in the current tree are removed from the updated file"
            else:
                sc += "The nodes <b>NOT</b> found in the current tree are kept unchanged"
        reply = MSG.wQuestion(self, 132, 'Saving on an already existing file',
                              """You are going to save into an existing file,
                              based on the current options you have and the target tree you want to save,
                              the result would be the following:<p>%s<p>
                              If this not the way you want the save to operate, please <i>abort</b> this
                              file selection and check the <i>Load/Save option</i> tab.
                              You still want to write on this file?""" % sc,
                              buttons=('Continue to save on existing file',
                                       'Abort save'))
        if reply:
            return None
        return 'User Abort'

    def path(self, index=None):
        if index is None:
            idx = self.treeview.currentIndex()
            p = str(self.model.filePath(self.proxy.mapToSource(idx)))
        else:
            p = str(self.model.filePath(self.proxy.mapToSource(index)))
        return p

    def clickedNodeAndLoad(self, index):
        self.clickedNode(index)
        if self.mode == SAVEMODE:
            self.save()
        else:
            self.load()

    def clickedNode(self, index):
        self.treeview.resizeColumnToContents(0)
        p = self.path(index)
        if os.path.isdir(p):
            f = ''
            d = p
            self.setCurrentDir(d)
        else:
            f = os.path.basename(self.path(index))
            d = os.path.dirname(self.path(index))
        ix = self.direntries.findText(d)
        if ix != -1:
            self.direntries.setCurrentIndex(ix)
        else:
            self.direntries.addItem(d)
            self.direntries.setCurrentIndex(self.direntries.findText(d))
        ix = self.fileentries.findText(f)
        if ix != -1:
            self.fileentries.setCurrentIndex(ix)
        else:
            self.fileentries.addItem(f)
            self.fileentries.setCurrentIndex(self.fileentries.findText(f))
        self.selecteddir = self.direntries.lineEdit().text()
        self.selectedfile = self.fileentries.lineEdit().text()

# --- last line
